@@ -3,6 +3,8 @@
 // Use is subject to license terms.
 //
 
+use std::ops;
+
 use serde::{de::DeserializeOwned, Serialize};
 use serde_json as json;
 
@@ -46,3 +48,14 @@ where
         self.0.detailed_show()
     }
 }
+
+impl<T> ops::Deref for Detailed<T>
+where
+    T: ops::Deref + DeserializeOwned + Serialize + Show,
+{
+    type Target = T::Target;
+
+    fn deref(&self) -> &Self::Target {
+        self.0.deref()
+    }
+}
