@@ -10,6 +10,7 @@ use std::io;
 use anyhow::Context;
 use console::Term;
 use dialoguer::{theme, Confirm, Input};
+use itertools::Itertools;
 use serde::{de::DeserializeOwned, Serialize};
 use structopt::StructOpt;
 // use structopt::clap;
@@ -23,6 +24,7 @@ use crate::Location;
 use crate::Output;
 
 use config::Config;
+use helper::AddLocation;
 
 mod config;
 mod helper;
@@ -163,8 +165,20 @@ enum Command {
     AddLocation {
         #[structopt(help = "State name")]
         state: v0::StateName,
-        #[structopt(help = "Location specification")]
-        location: Location,
+        #[structopt(
+            help = "Location specification",
+            conflicts_with = "cluster",
+            required_unless = "cluster"
+        )]
+        location: Option<Location>,
+        #[structopt(
+            help = "Add locations from this cluster",
+            long,
+            short,
+            conflicts_with = "location",
+            required_unless = "location"
+        )]
+        cluster: Option<v0::ClusterName>,
         #[structopt(help = "Wait until new location is ready", long)]
         wait: bool,
     },
@@ -407,8 +421,17 @@ impl Cli {
             Command::AddLocation {
                 state,
                 location,
+                cluster,
                 wait,
-            } => statehub.add_location(state, location, wait).await,
+            } => {
+                let location = location.map(AddLocation::FromLocation);
+                let cluster = cluster.map(AddLocation::FromCluster);
+                if let Some(add_location) = location.or(cluster) {
+                    statehub.add_location(state, add_location, wait).await
+                } else {
+                    anyhow::bail!("Need to specify either location or --cluster");
+                }
+            }
             Command::RemoveLocation { state, location } => {
                 statehub.remove_location(state, location).await
             }
@@ -536,11 +559,12 @@ impl StateHub {
     }
 
     async fn show_state(self, state: &v0::StateName) -> anyhow::Result<()> {
-        self.api
-            .get_state(state)
-            .await
-            .map(Detailed)
-            .handle_output(&self.stdout, self.json)
+        let state = self.api.get_state(state).await.map(Detailed)?;
+        if let Ok(clusters) = self.api.get_all_clusters().await {
+            (state, clusters).handle_output(&self.stdout, self.json)
+        } else {
+            state.handle_output(&self.stdout, self.json)
+        }
     }
 
     async fn list_states(&self) -> anyhow::Result<()> {
@@ -582,7 +606,8 @@ impl StateHub {
         let cluster = self
             .api
             .register_cluster(&cluster, provider, &locations)
-            .await?;
+            .await
+            .map(Quiet)?;
 
         self.inform(format_args!(
             "Registering {:#} cluster {} in {}",
@@ -613,11 +638,12 @@ impl StateHub {
     }
 
     async fn show_cluster(&self, name: v0::ClusterName) -> anyhow::Result<()> {
-        self.api
-            .get_cluster(&name)
-            .await
-            .map(Detailed)
-            .handle_output(&self.stdout, self.json)
+        let cluster = self.api.get_cluster(&name).await.map(Detailed)?;
+        if let Ok(states) = self.api.get_all_states().await {
+            (cluster, states).handle_output(&self.stdout, self.json)
+        } else {
+            cluster.handle_output(&self.stdout, self.json)
+        }
     }
 
     async fn unregister_cluster(&self, name: v0::ClusterName, force: bool) -> anyhow::Result<()> {
@@ -639,18 +665,33 @@ impl StateHub {
     async fn add_location(
         &self,
         state: v0::StateName,
-        location: Location,
+        add_location: AddLocation,
         wait: bool,
     ) -> anyhow::Result<()> {
-        let state = self.api.get_state(&state).await?;
-        if state.is_available_in(&location) {
-            self.inform(format_args!(
-                "State {} is already available in {:#}",
-                state, location
-            ))?;
-        } else {
-            self.inform(format_args!("Extending state {} to {:#}", state, location))?;
-            self.add_location_helper(&state, &location, wait).await?;
+        let locations = match add_location {
+            AddLocation::FromLocation(location) => vec![location],
+            AddLocation::FromCluster(cluster) => {
+                let locations = self.api.get_cluster(&cluster).await?.all_locations();
+                self.inform(&format!(
+                    "Cluster {} available in {}",
+                    cluster,
+                    locations.iter().map(ToString::to_string).join(" and ")
+                ))?;
+                locations
+            }
+        };
+
+        for location in locations {
+            let state = self.api.get_state(&state).await?;
+            if state.is_available_in(&location) {
+                self.inform(format_args!(
+                    "State {} is already available in {:#}",
+                    state, location
+                ))?;
+            } else {
+                self.inform(format_args!("Extending state {} to {:#}", state, location))?;
+                self.add_location_helper(&state, &location, wait).await?;
+            }
         }
         Ok(())
     }
@@ -919,3 +960,49 @@ where
         Ok(())
     }
 }
+
+impl HandleOutput for (Detailed<Output<v0::State>>, Output<Vec<v0::Cluster>>) {
+    fn handle_output(self, stdout: &Term, json: bool) -> anyhow::Result<()> {
+        let (state, clusters) = self;
+        let state_locations = state.all_locations();
+
+        state.handle_output(stdout, json)?;
+
+        let visible_clusters = clusters
+            .iter()
+            .filter(|cluster| {
+                cluster
+                    .all_locations()
+                    .iter()
+                    .any(|location| state_locations.contains(location))
+            })
+            .map(ToString::to_string)
+            .join(" ");
+
+        stdout.write_line(&format!("Visible clusters:\n    {}", visible_clusters))?;
+        Ok(())
+    }
+}
+
+impl HandleOutput for (Detailed<Output<v0::Cluster>>, Output<Vec<v0::State>>) {
+    fn handle_output(self, stdout: &Term, json: bool) -> anyhow::Result<()> {
+        let (cluster, states) = self;
+        let cluster_locations = cluster.all_locations();
+
+        cluster.handle_output(stdout, json)?;
+
+        let visible_states = states
+            .iter()
+            .filter(|state| {
+                state
+                    .all_locations()
+                    .iter()
+                    .any(|location| cluster_locations.contains(location))
+            })
+            .map(ToString::to_string)
+            .join(" ");
+
+        stdout.write_line(&format!("Visible states:\n    {}", visible_states))?;
+        Ok(())
+    }
+}
